Cormas_Model subclass: #FireAutomata
	instanceVariableNames: 'theFireAutomata_Cells trees '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Cormas-FireAutomata'!


!FireAutomata methodsFor: 'acces'!

theFireAutomata_Cells
	^theFireAutomata_Cells!

theFireAutomata_Cells: x 
	theFireAutomata_Cells := x! !

!FireAutomata methodsFor: 'init'!

init53
	self initCells: #init53.
	self initCharts!

init58
	self initCells: #init58.
	self initCharts! !

!FireAutomata methodsFor: 'control'!

stepAsynchronously: t 
	super stepAsynchronously: t.
	super updateCharts: t.!

stepSynchronously: t 
	super stepSynchronously: t.
	super updateCharts: t.! !

!FireAutomata methodsFor: 'graphiques'!

trees
^trees isNil
ifTrue: [trees := List new]
ifFalse: [trees]!

trees: x
trees := x!

treesData
"return the data to be plotted with the trees chart"
	^(self theFireAutomata_Cells select: [:c | c state = #tree]) size / self theFireAutomata_Cells size! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

FireAutomata class
	instanceVariableNames: ''!


!FireAutomata class methodsFor: 'description'!

aboutIt
	| txt |
	txt := 'This model illustrates how the principles of cellular automata are 
implemented in Cormas. 

The spatial entity of the model (the cell), named FireAutomata_Cell, 
can take four states: #fire (red); #tree (green); #ash (grey); #empty (white). 
The initial state of each cell of the spatial grid is either set to #tree with a 
probability p or to #empty with a probability 1-p. 

One cell is set on fire, and then the spreading of the fire, defined in the 
cell transition function, occurs. The transition function is the following: 
a cell being a tree at time t-1 will become on fire at time t if at least 
one of its 4 neighbours (North, East, South, West) is on fire at time t-1. 
The cells being on fire will become ash at the next timestep, the cells 
being ash will become empty at the next timestep. 

The probability to observe a resteint fire is high if p is lower than 0.55, 
whereas when p is greater than 0.55, a global fire is likely to occur. 
This percolation threshold characterizes cellular automata which are 
representing diffusion processes.'
				asText emphasizeAllWith: #(#bold).
	ComposedTextView open: txt asValue
		label: 'About ' , self name
		icon: (Icon constantNamed: #workspace)
		extent: 500 @ 430!

howToRunIt

| txt |
txt := '1. Prepare the visualisation of the spatial grid\' withCRs asText emphasizeAllWith: #(#bold #large).
txt := txt, '1.1 Click on the first icon of the ' asText, ('Visualisation ' asText emphasizeAllWith: #color -> ColorValue red), 'area to open the standard spatial grid.\' withCRs asText.
txt := txt, '1.2 In the ' asText, ('Topology ' asText emphasizeAllWith: #italic), 'menu, clicking on the ' asText, ('Grid size ' asText emphasizeAllWith: #italic), 'item opens a window allowing you to choose the number of lines and columns. For example, choose 30 * 30.\' withCRs asText.
txt := txt, '1.3 Check the border condition: ' asText, ('Topology -> Grid boundaries -> closed' asText emphasizeAllWith: #italic), '.\' withCRs asText.
txt := txt, '1.4 Right-click within the spatial grid window to select the ' asText, ('pov ' asText emphasizeAllWith: #italic), 'view for ' asText, ('FireAutomata_Cell ' asText emphasizeAllWith: #italic), '.\' withCRs asText.
txt := txt, '1.5 To display the legend of this viewpoint: ' asText, ('Tools -> Display legends.\\' withCRs asText emphasizeAllWith: #italic).

txt := txt, ('2. Prepare the initial spatial grid\' withCRs asText emphasizeAllWith: #(#bold #large)).
txt := txt, '2.1 Click on the ' asText, ('Initialize ' asText emphasizeAllWith: #italic), 'button of the ' asText, ('Simulation ' asText emphasizeAllWith: #color -> ColorValue red),  'area.\' withCRs asText.
txt := txt, '2.2 Select the ' asText, ('init53 ' asText emphasizeAllWith: #italic), 'and ' asText, ('stepSynchronously: ' asText emphasizeAllWith: #italic), 'methods, then validate by clicking on ' asText, ('Apply and close.\' asText emphasizeAllWith: #italic) withCRs asText.
txt := txt, '2.3 From the spatial grid menu, use ' asText, ('Tools -> Click to... -> Change attribute... -> state' asText emphasizeAllWith: #italic), ', enter ' asText, ('fire ' asText emphasizeAllWith: #italic), 'and validate. The cursor takes a new shape. Click on a cell to change manually its state to fire, then switch back to the mouse select mode with ' asText, ('Tools -> Click to... -> Select ' asText emphasizeAllWith: #italic), '.\\' withCRs asText. 

txt := txt, ('3. Run the model\' withCRs asText emphasizeAllWith: #(#bold #large)).
txt := txt, '3.1 Enter 50 in the ' asText, ('Final time ' asText emphasizeAllWith: #italic), 'input field of the ' asText,  ('Simulation ' asText emphasizeAllWith: #color -> ColorValue red),  'area.\' withCRs asText.
txt := txt, '3.2 Click on the ' asText, ('Run ' asText emphasizeAllWith: #italic), 'button of the ' asText,  ('Simulation ' asText emphasizeAllWith: #color -> ColorValue red),  'area.\\' withCRs asText.

txt := txt, ('4. Have a look at the charts\' withCRs asText emphasizeAllWith: #(#bold #large)).
txt := txt, '4.1 Click on the third icon of the ' asText, ('Visualisation ' asText emphasizeAllWith: #color -> ColorValue red), 'area to open the charts window.\\' withCRs asText.

txt := txt, ('5. Run alternative versions of the model\' withCRs asText emphasizeAllWith: #(#bold #large)).
txt := txt, '5.1 Select the ' asText, ('init58 ' asText emphasizeAllWith: #italic), 'and ' asText, ('stepSynchronously: ' asText emphasizeAllWith: #italic), 'methods, then run the model again.\' withCRs asText.
txt := txt, '5.2 Select the ' asText, ('init53 ' asText emphasizeAllWith: #italic), 'and ' asText, ('stepAsynchronously: ' asText emphasizeAllWith: #italic), 'methods, then run the model again.\' withCRs asText.
ComposedTextView open: txt asValue label: ('How to run ', self name, '     step-by-step instructions') icon: (Icon constantNamed: #workspace) extent: 620@550! !

SpatialEntity_CA subclass: #FireAutomata_Cell
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Cormas-FireAutomata'!


!FireAutomata_Cell methodsFor: 'controle'!

newState
	self state = #tree ifTrue: [(self neighbourhood contains: [:a | a state = #fire])
			ifTrue: [self bufferState: #fire]
			ifFalse: [self bufferState: #tree]].
	self state = #fire ifTrue: [self bufferState: #ash].
	self state = #ash ifTrue: [self bufferState: #empty].
	self state = #empty ifTrue: [self bufferState: #empty]! !

!FireAutomata_Cell methodsFor: 'init'!

init50
	Cormas random < 0.5 
		ifTrue: [self state: #tree] 
		ifFalse: [self state: #empty]!

init53
	Cormas random < 0.53
		ifTrue: [self state: #tree] 
		ifFalse: [self state: #empty]!

init58
	Cormas random < 0.58 
		ifTrue: [self state: #tree] 
		ifFalse: [self state: #empty]!

initEmpty
self state: #empty! !

!FireAutomata_Cell methodsFor: 'pov'!

pov
"^#empty ^#tree ^#fire ^#ash"
^ self state! !

